/*
 * Copyright 2014 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * Authors:
 *   Simon Lukasik <slukasik@redhat.com>
 *
 */

#pragma once
#ifndef _OSCAP_XCCDF_POLICY_REPORT_CB_PRIV_H
#define _OSCAP_XCCDF_POLICY_REPORT_CB_PRIV_H

#include "common/util.h"
#include "public/xccdf_policy.h"


/**
 * Callback structure with callback function and usr data (optional)
 * After rule evaluation action will be called the callback with user data.
 */
struct reporter;

struct reporter *reporter_new(char *report_type, void *output_func, void *usr);

int reporter_send_simple(struct reporter *reporter, void *data);


#endif
